/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view.AccuracyTests.Enh3;

import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;

import junit.framework.TestCase;

import com.topcoder.rules.dependencymap.view.Dependency;
import com.topcoder.rules.dependencymap.view.DependencyRenderer;
import com.topcoder.rules.dependencymap.view.DiagramRenderer;
import com.topcoder.rules.dependencymap.view.FixedGridLayoutManager;
import com.topcoder.rules.dependencymap.view.RendererException;
import com.topcoder.rules.dependencymap.view.ShapeFactory;
import com.topcoder.rules.dependencymap.view.AccuracyTests.GraphicsHelper;
import com.topcoder.rules.dependencymap.view.AccuracyTests.MockDiagramRenderer;
import com.topcoder.rules.dependencymap.view.AccuracyTests.MockGraphics;

/**
 * Tets for the enhancement 3, using the configuration manager
 *
 * @author cucu
 * @version 1.0
 */
public class Enh3Tests extends TestCase {
    private DependencyRenderer renderer = null;
    private MockDiagramRenderer diagramRenderer = null;
    private GraphicsHelper gh = null;
    private Dependency a = null;
    private Dependency b = null;
    private Dependency c = null;
    private static final String FILENAME = "test_files/enh2.svg";

    /**
     * Set up for each test.
     */
    public void setUp() {
        renderer = new DependencyRenderer();
        diagramRenderer = new MockDiagramRenderer();
        renderer.addDiagramRenderer(diagramRenderer);

        a = new Dependency("A");
        b = new Dependency("B");
        c = new Dependency("C");
        a.addDependant(b);
        a.addDependant(c);

        gh = new GraphicsHelper(new ShapeFactory(), new MockGraphics(null), 10, 10, new Insets(10, 10, 10, 10));
    }

    /**
     * Draws a simple diagram where B and C depends on A
     */
    private void drawDiagram() throws RendererException {
        gh.setShapeSizeTo("A");
        gh.drawShape(false,2,1, "A");
        gh.drawShape(false,1,2, "B");
        gh.drawShape(false,3,2, "C");
        gh.drawArrow(2,1,1,2);
        gh.drawArrow(2,1,3,2);
        gh.getGraphics().complete();
    }

    /**
     *  Test if the colors are properly used
     */
    public void testColor1() throws RendererException {
        renderer.setLayoutManager(new FixedGridLayoutManager("com.topcoder.rules.dependencymap.view.testColor1"));
        renderer.render(a);

        gh.setColors(Color.RED, Color.YELLOW, Color.BLUE, Color.BLACK, Color.CYAN);
        compare("testColor1", gh, diagramRenderer);
    }

    /**
     *  Test if the font is correctly set.
     */
    public void testFont1() throws RendererException {
        renderer.setLayoutManager(new FixedGridLayoutManager("com.topcoder.rules.dependencymap.view.testFont1"));
        renderer.render(a);
        gh.setFont(new Font("Serif", Font.BOLD, 17));

        compare("testFont1", gh, diagramRenderer);
    }
    /**
     *  Test overriding the default padding and margin.
     */
    public void testPaddingAndMargin1() throws RendererException {
        Insets insets = new Insets(5, 5, 5, 5);
        renderer.setLayoutManager(new FixedGridLayoutManager("com.topcoder.rules.dependencymap.view.testPaddingAndMargin1"));
        renderer.render(a);

        gh = new GraphicsHelper(new ShapeFactory(), new MockGraphics(), 37, 49, insets);

        compare("testPaddingAndMargin1", gh, diagramRenderer);
    }

    /**
     *  Test overriding all the settings.
     */
    public void testComplete1() throws RendererException {
        Insets insets = new Insets(5, 5, 5, 5);
        renderer.setLayoutManager(new FixedGridLayoutManager("com.topcoder.rules.dependencymap.view.testComplete1"));
        renderer.render(a);

        gh = new GraphicsHelper(new ShapeFactory(), new MockGraphics(), 37, 49, insets);

        gh.setColors(Color.RED, Color.YELLOW, Color.BLUE, Color.BLACK, Color.CYAN);
        gh.setFont(new Font("Serif", Font.BOLD, 17));

        compare("testComplete1", gh, diagramRenderer);
    }

    /**
     * Save the expected and the resulting SVG. Please compare them visually.
     */
    protected void compare(String test, GraphicsHelper gh, DiagramRenderer dr) throws RendererException {
        drawDiagram();
        ((MockGraphics)dr.getGraphics(a,1,1)).save("test_files\\enh3\\" + test + "_result");
        gh.getGraphics().save("test_files\\enh3\\" + test + "_expected");
    }

}